<?php /*

 ocPortal
 Copyright (c) ocProducts, 2004-2012

 See text/EN/licence.txt for full licencing information.


 NOTE TO PROGRAMMERS:
   Do not edit this file. If you need to make changes, save your changed file to the appropriate *_custom folder
   **** If you ignore this advice, then your website upgrades (e.g. for bug fixes) will likely kill your changes ****

*/

/**
 * @license		http://opensource.org/licenses/cpal_1.0 Common Public Attribution License
 * @copyright	ocProducts Ltd
 * @package		ocf_forum
 */

class Hook_rss_ocf_forumview
{

	/**
	 * Standard modular run function for RSS hooks.
	 *
	 * @param  string			A list of categories we accept from
	 * @param  TIME			Cutoff time, before which we do not show results from
	 * @param  string			Prefix that represents the template set we use
	 * @set    RSS_ ATOM_
	 * @param  string			The standard format of date to use for the syndication type represented in the prefix
	 * @param  integer		The maximum number of entries to return, ordering by date
	 * @return ?array			A pair: The main syndication section, and a title (NULL: error)
	 */
	function run($_filters,$cutoff,$prefix,$date_string,$max)
	{
		if (!addon_installed('ocf_forum')) return NULL;

		if (get_forum_type()!='ocf') return NULL;
		if (!has_actual_page_access(get_member(),'forumview')) return NULL;

		$filters=ocfilter_to_sqlfragment($_filters,'t_forum_id','f_forums','f_parent_forum','t_forum_id','id',true,true,$GLOBALS['FORUM_DB']); // Note that the parameters are fiddled here so that category-set and record-set are the same, yet SQL is returned to deal in an entirely different record-set (entries' record-set)

		$rows=$GLOBALS['FORUM_DB']->query('SELECT * FROM '.$GLOBALS['FORUM_DB']->get_table_prefix().'f_topics WHERE t_cache_last_time>'.strval($cutoff).((!has_specific_permission(get_member(),'see_unvalidated'))?' AND t_validated=1 ':'').' AND '.$filters.' ORDER BY t_cache_last_time DESC',$max);
		$categories=collapse_2d_complexity('id','f_name',$GLOBALS['FORUM_DB']->query('SELECT id,f_name FROM '.$GLOBALS['FORUM_DB']->get_table_prefix().'f_forums WHERE f_cache_num_posts>0'));

		$content=new ocp_tempcode();
		foreach ($rows as $row)
		{
			if (((!is_null($row['t_forum_id'])) || ($row['t_pt_to']==get_member())) && (has_category_access(get_member(),'forums',strval($row['t_forum_id']))))
			{
				$id=strval($row['id']);
				$author=$row['t_cache_first_username'];

				$news_date=date($date_string,$row['t_cache_first_time']);
				$edit_date=date($date_string,$row['t_cache_last_time']);
				if ($edit_date==$news_date) $edit_date='';

				$news_title=xmlentities($row['t_cache_first_title']);
				$_summary=get_translated_tempcode($row['t_cache_first_post'],$GLOBALS['FORUM_DB']);
				$summary=xmlentities($_summary->evaluate());
				$news='';

				$category=array_key_exists($row['t_forum_id'],$categories)?$categories[$row['t_forum_id']]:do_lang('NA');
				$category_raw=strval($row['t_forum_id']);

				$view_url=build_url(array('page'=>'topicview','id'=>$row['id']),get_module_zone('forumview'),NULL,false,false,true);

				if ($prefix=='RSS_')
				{
					$if_comments=do_template('RSS_ENTRY_COMMENTS',array('_GUID'=>'f5dd7ba612b989bba5e2d496da5bf161','COMMENT_URL'=>$view_url,'ID'=>strval($row['id'])));
				} else $if_comments=new ocp_tempcode();

				$content->attach(do_template($prefix.'ENTRY',array('VIEW_URL'=>$view_url,'SUMMARY'=>$summary,'EDIT_DATE'=>$edit_date,'IF_COMMENTS'=>$if_comments,'TITLE'=>$news_title,'CATEGORY_RAW'=>$category_raw,'CATEGORY'=>$category,'AUTHOR'=>$author,'ID'=>$id,'NEWS'=>$news,'DATE'=>$news_date)));
			}
		}

		require_lang('ocf');
		return array($content,do_lang('SECTION_FORUMS'));
	}

}


